# -*- coding: cp1252 -*-
'''
GTKRawGallery module for curve tool;
copyright  2013 Daniele Isca
license: GNU GPL v.3 (see COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''

import pygtk
import gtk
import cairo
import math
import numpy as np
import sys


W = 250
H = 250

class Curve(object):
    def __init__(self, gallery, da):
        self.gallery = gallery
        self.points = []
        self.p = None
        self.da = da
        self.da.set_size_request(W,H)
        self.da.connect('expose_event', self.on_expose)
        self.da.add_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON1_MOTION_MASK | gtk.gdk.POINTER_MOTION_HINT_MASK)
        self.da.connect('motion_notify_event', self.on_motion_event)
        self.da.connect('button_press_event', self.on_button_press)
        da.get_parent().show_all()

    def reset(self, obj=None):
        if self.da.window:
            self.points = []
            cr = self.da.window.cairo_create()
            self.draw(cr, W,H)
            cr.set_line_width(.8)
            cr.set_source_rgb(1, 1, 1)
            cr.move_to(0,H)
            cr.line_to(W,0)
            cr.arc(1, H-1, 1, 0.0, 2 * math.pi)
            cr.arc(W-1, 1, 1, 0.0, 2 * math.pi)
            cr.stroke()
     
    def on_expose(self, area, event):
        cr = area.window.cairo_create()
        # Restrict Cairo to the exposed area; avoid extra work
        cr.rectangle(event.area.x, event.area.y, event.area.width, event.area.height)
        cr.clip()
        self.draw(cr, W, H)
        cr.set_line_width(1)
        cr.set_source_rgb(1, 1, 1)
        if not self.points:
            cr.move_to(0, H)
            cr.line_to(W, 0)
            cr.stroke()
        else:
            for item in self.points:
                cr.arc(item[0], item[1], 2, 0.0, 2 * math.pi)
                cr.stroke()
            cr.move_to(0, H)
            y_list = self.get_poly()
            for a,b in zip(xrange(0, W+1, 2), y_list):
                if b < 0:
                    b = 1
                elif b > H:
                    b = H - 1
                cr.line_to(a, b)
            cr.stroke()

    def draw(self, cr, width, height):
        cr.set_source_rgb(.3, .3, .3)
        cr.rectangle(0, 0, width, height)
        cr.stroke()
        cr.set_source_rgb(.2,.2,.2)
        cr.rectangle(1, 1, width - 2, height - 2)
        cr.fill()
        cr.set_source_rgb(.3, .3, .3)
        cr.set_line_width(2)
        cr.move_to(width / 4, 1)
        cr.line_to(width / 4, height - 1)
        cr.move_to(width / 2, 1)
        cr.line_to(width / 2, height - 1)
        cr.move_to(width * 3/4, 1)
        cr.line_to(width * 3/4, height - 1)
        cr.move_to(1, height / 4)
        cr.line_to(width - 1, height / 4)
        cr.move_to(1, height / 2)
        cr.line_to(width - 1, height / 2)
        cr.move_to(1, height* 3/4)
        cr.line_to(width - 1, height * 3/4)
        cr.stroke()
       
    def on_button_press(self, obj, event):
        for item in self.points:
            if (item[0] - 20 <= event.x <= item[0] + 20):
                self.points.remove(item)
        self.points.append((event.x, event.y))
        cr = obj.window.cairo_create()
        self.draw(cr, W,H)
        cr.set_line_width(1)
        cr.set_source_rgb(1, 1, 1)
        for item in self.points:
            cr.arc(item[0], item[1], 2, 0.0, 2 * math.pi)
            cr.stroke()
        cr.move_to(0, H)
        y_list = self.get_poly()
        for a,b in zip(xrange(0, W+1, 2), y_list):
            if b < 0:
                b = 1
            elif b > H:
                b = H - 1
            cr.line_to(a, b)
        cr.stroke()

    def on_motion_event(self, area, event):
        if 0 < event.x < W and 0 < event.y < H:
            for item in self.points:
                if (item[0] - 20 <= event.x <= item[0] + 20) and (item[1] - 20 <= event.y <= item[1] + 20):
                    self.points.remove(item)
                    self.points.append((event.x, event.y))
            cr = area.window.cairo_create()
            cr.rectangle(0, 0, W, H)
            cr.clip()
            self.draw(cr, W, H)
            cr.set_line_width(1)
            cr.set_source_rgb(1, 1, 1)
            for item in self.points:
                cr.arc(item[0], item[1], 2, 0.0, 2 * math.pi)
                cr.stroke()
            cr.move_to(0, H)
            y_list = self.get_poly()
            for a,b in zip(xrange(0, W + 1, 2), y_list):
                if b < 0:
                    b = 1
                elif b > H:
                    b = H - 1
                cr.line_to(a, b)
            cr.stroke()

    def get_poly(self, data=True):
        points = sorted(self.points)
        x = [0] + [i[0] for i in points] + [W]
        y = [H] + [ i[1] for i in points] + [0]
        x1 = [i / float(W) for i in x]
        y1 = [1 - i/float(H) for i in y]
        self.p = np.polyfit(x1, y1, len(x1)-1)
        if data:
            p = np.polyfit(x, y, len(x)-1)
            return np.polyval(p, xrange(0, W+1, 2)).tolist()

    def get_coeffs(self):
        if self.points:
            return self.p
        return np.array([1])
        
    
