# -*- coding: cp1252 -*-
'''
description: GTKRawGallery module for Exiftool batch mode interface;
copyright  2013 Daniele Isca
license: GNU GPL v.3 (see file COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''

import subprocess
import os
import json
import sys
from threading import Lock

if sys.platform.startswith('win'):
    sentinel = "{ready}\r\n"
    CREATIONFLAGS = 134217728
else:
    sentinel = "{ready}\n"
    CREATIONFLAGS = 0

class ExifTool(object):

    def __init__(self, executable, argfile):
        self.lock = Lock()
        self.argfile = argfile
        self.executable = executable
        self.__enter__()

    def __enter__(self):
        self.fd = open(self.argfile,'wb',0)
        self.process = subprocess.Popen([self.executable, "-stay_open", "True", "-@", self.argfile], stdout=subprocess.PIPE, creationflags = CREATIONFLAGS)

    def  _exit(self):
        with self.lock:
            self.fd.write("-stay_open\nFalse\n")
            self.fd.close()

    def execute(self, *args):
        with self.lock:
            args = args + ("-execute\n",)
            self.fd.write(str.join("\n", args))
            output = ""
            fd = self.process.stdout.fileno()
            while not output.endswith(sentinel):
                output += os.read(fd,4096)
            return output[:-len(sentinel)]

    def get_metadata(self, format_='string', *args):
        if format_ == 'json':
            return json.loads(self.execute(*args))[0]
        return self.execute(*args)
