# -*- coding: utf-8 -*-
'''
GTKRawGallery DropBox uploader;
copyright © 2013 Daniele Isca
license: GNU GPL v.3 (see COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''
import gtk
import pygtk
import os.path
import os
import traceback
import sys
import time
import dropbox
import webkit
from gtkrg_profiles import dbx_data

def _decode(data):
    return ''.join([chr(i) for i in data])


class Publisher:
    def __init__(self, gallery, image_list, gtkrg_datadir):
        self.gallery = gallery
        self.image_list = image_list
        self.stop = False
        self.uploading = False
        self.token_key = None
        self.token_secret = None
        self.session = None
        wtree = gtk.Builder()
        wtree.add_from_file(os.path.join(gtkrg_datadir, 'publisher.ui'))
        self.initialize_widgets(wtree)
        self.boxmodel1 = gtk.ListStore(str)
        cell = gtk.CellRendererText()
        self.combobox1.set_model(self.boxmodel1)
        self.combobox1.pack_start(cell, True)
        self.combobox1.add_attribute(cell, 'text', 0)
        self.window1.set_icon_from_file(self.gallery.get_icon_path('dropbox.png'))
        self.make_dp_treeview()
        self.window1.set_title(_('Upload into DropBox'))
        user = ''
        if self.gallery.settings.has_key('dropbox_user'):
            user = self.gallery.settings['dropbox_user']
            if self.gallery.settings.has_key('dropbox_tokens') and self.gallery.settings['dropbox_tokens'].has_key(user):
                self.token_key = self.gallery.settings['dropbox_tokens'][user]['key']
                self.token_secret = self.gallery.settings['dropbox_tokens'][user]['secret']
        if not self.gallery.settings.has_key('dropbox_tokens'):
                self.gallery.settings['dropbox_tokens'] = {}
        self.checkbutton1.set_property('visible', False)
        self.checkbutton2.set_property('visible', False)
        self.checkbutton3.set_property('visible', False)
        self.combobox2.set_property('visible', False)
        self.label5.set_property('visible', False)
        self.label4.set_property('visible', False)
        self.label6.set_text(_('Dropbox user:'))
        self.radiobutton2.set_label(_('New folder'))
        self.radiobutton2.set_tooltip_text(_('creates a new destination folder'))
        self.entry1.set_tooltip_text(_('insert the folder name'))
        self.entry4.set_property('visible', False)
        self.radiobutton1.set_label(_('Upload to:'))
        self.boxmodel3 = gtk.ListStore(str)
        cell = gtk.CellRendererText()
        self.combobox3.set_model(self.boxmodel3)
        self.combobox3.pack_start(cell, True)
        self.combobox3.add_attribute(cell, 'text', 0)
        # loads webview UI
        wview = gtk.Builder()
        wview.add_from_file(os.path.join(gtkrg_datadir, 'webview.ui'))
        self.browser = wview.get_object('window1')
        self.scrolled_window = wview.get_object('scrolledwindow1')
        self.wview_button = wview.get_object('button1')
        self.web_view = webkit.WebView()
        self.scrolled_window.add(self.web_view)
        self.connect_signals()
        self.browser.set_title(_('Log into Dropbox'))
        self.browser.set_icon_from_file(self.gallery.get_icon_path('dropbox.png'))
        self.browser.set_default_size(800, 600)
        self.browser.set_keep_above(True)
        if self.gallery.settings.has_key('dropbox_tokens'):
            self.combobox3.handler_block(self.id3)
            for key in sorted(self.gallery.settings['dropbox_tokens'].keys()):
                self.boxmodel3.append([key])
            for row in self.boxmodel3:
                if row[0] == user:
                    self.combobox3.set_active_iter(row.iter)
                    break
            else:
                self.combobox3.set_active(0)
            self.combobox3.handler_unblock(self.id3)
        if self.gallery.window3.get_property('visible'):
            self.window1.set_transient_for(self.gallery.window3)
        elif self.gallery.window2.get_property('visible'):
            self.window1.set_transient_for(self.gallery.window2)
        else:
            self.window1.set_transient_for(self.gallery.window1)

    def load_started(self, *obj):
        if self.browser.window:
            self.browser.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))

    def load_finished(self, *obj):
        if self.browser.window:
            self.browser.window.set_cursor(None)

    def load_error(self, *obj):
        self.web_view.load_string(_('<div>Page Loading Error!</div>'), 'text/html', 'UTF-8', '#')
        
    def hide_webview(self, obj, event=None):
        self.browser.hide()
        return True

    def make_dp_treeview(self):
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
        renderer_pix = gtk.CellRendererPixbuf()
        column_pix = gtk.TreeViewColumn("Preview", renderer_pix, pixbuf=0)
        column_pix.set_cell_data_func (renderer_pix, self.cell_data_function, None)
        self.treeview1.append_column(column_pix)
        cell = gtk.CellRendererText()                                                                           
        column_item1 = gtk.TreeViewColumn("Filename", cell, text=1)
        column_item1.set_cell_data_func (cell, self.cell_data_function, None)
        column_item1.set_resizable(True)                                                                                         
        self.treeview1.append_column(column_item1)
        cell = gtk.CellRendererText()
        column_status = gtk.TreeViewColumn("Status", cell, text=2)
        column_status.set_cell_data_func (cell, self.cell_data_function, None)
        self.treeview1.append_column(column_status)
        self.treeview1.set_model(self.model)

    def cell_data_function(self, column, cell, model, _iter, data=None):
        value = model.get_value(_iter, 2)
        if value == None:
            color = self.treeview1.rc_get_style().base[gtk.STATE_NORMAL].to_string()
        elif value == 'uploading...':
            color = '#CD7300'
        elif value == 'error!':
            color = 'red'
        else:
            color = self.treeview1.rc_get_style().base[gtk.STATE_NORMAL].to_string()
        cell.set_property('cell_background', color)
        
    def initialize_widgets(self, tree):
        for widget in tree.get_objects():
            name = gtk.Buildable.get_name(widget)
            if name and not hasattr(self, name):
                setattr(self, name, widget)

    def connect_signals(self):
        self.button1.connect('clicked', self.destroy)
        self.button2.connect('clicked', self.upload)
        self.window1.connect('delete_event', self.destroy)
        self.window1.connect('key_press_event', self.on_key_press_event)
        self.button5.connect('clicked', self.change_user_account)
        self.id3 = self.combobox3.connect('changed', self.change_user_account)
        self.radiobutton1.connect('toggled', self.on_radiobutton_toggled)
        self.web_view.connect('load_error', self.load_error)
        self.web_view.connect('load_finished', self.load_finished)
        self.web_view.connect('load_started', self.load_started)
        self.browser.connect('delete_event', self.hide_webview)
        self.wview_button.connect('clicked', self.get_token_cb)

    def on_radiobutton_toggled(self, obj):
        self.entry1.set_sensitive(not obj.get_active())
        self.combobox1.set_sensitive(obj.get_active())

    def on_key_press_event(self, obj, event):
        keyname = gtk.gdk.keyval_name(event.keyval)
        if keyname == 'Delete' and len(self.model):
            selection = self.treeview1.get_selection()
            model, iter_ = selection.get_selected()
            if iter_:
                model.remove(iter_)

    def populate_dir_box(self, dirs, text=None):
        self.boxmodel1.clear()
        for dir_ in sorted(dirs):
            self.boxmodel1.append([dir_])
        if text:
            for row in self.boxmodel1:
                if row == text:
                    self.combobox1.set_active_iter(row.iter)
                    break
        else:
            self.combobox1.set_active(0)

    def upload(self, obj=None):
        if len(self.model):
            if self.radiobutton1.get_active():
                remote_dir = self.combobox1.get_active_text()
            else:
                remote_dir = self.entry1.get_text() or ''
                if not remote_dir.startswith('/'):
                    remote_dir = '/' + remote_dir
                    remote_dir = os.path.normpath(remote_dir).replace('\\','/')
                if self.client:
                    try:
                        self.client.file_create_folder(remote_dir)
                    except dropbox.rest.ErrorResponse, e:
                        sys.stderr.write(e.error)
            for row in self.model:
                self.model.set_value(row.iter, 2, '')
            if self.client:
                self.button2.set_sensitive(False)
                self.stop = False
                self.uploading = True
                self.upload_to_dropbox(remote_dir)
                self.stop = False
                self.uploading = False
                self.button2.set_sensitive(True)
            else:
                self.label3.set_text(_('not logged!'))
                self.dropbox_login()

    def upload_to_dropbox(self, remote_dir):
        fraction = 0.99 / len(self.model)
        self.progressbar1.set_fraction(0)
        self.progressbar1.show()
        while gtk.events_pending():
            gtk.main_iteration(False)
        for row in self.model:
            if self.stop:
                break
            image = self.get_source_file(row[1])
            self.progressbar1.set_text(_('Uploading %s ...') % row[1])
            self.model.set_value(row.iter, 2, 'uploading...')
            while gtk.events_pending():
                gtk.main_iteration(False)
            try:
                f = open(image, 'rb')
                resp = self.client.put_file(remote_dir+ '/' + row[1], f)
                print resp
                f.close()
                self.model.set_value(row.iter, 2, 'completed!')
            except dropbox.rest.ErrorResponse, e:
                self.model.set_value(row.iter, 2, 'error!')
                sys.stderr.write(e.error)
            except:
                self.model.set_value(row.iter, 2, 'error!')
                sys.stderr.write(traceback.format_exc())
            self.progressbar1.set_fraction(self.progressbar1.get_fraction() + fraction)
            self.treeview1.scroll_to_cell(self.model.get_path(row.iter))
            while gtk.events_pending():
                gtk.main_iteration(False)
        time.sleep(1)
        self.progressbar1.hide()

    def get_source_file(self, basename):
        return self.gallery.get_real_path(self.rawpath[basename])
                           
    def refresh_treeview(self):
        self.rawpath = {}
        self.model.clear()
        for item in self.gallery.selected:
            basename = os.path.basename(self.gallery.get_real_path(item))
            self.rawpath[basename] = item
            if self.image_list[0] == self.gallery.image28:
                pixbuf = self.gallery.scale_thumbnail(self.image_list[0].get_pixbuf().copy())
            else:
                pixbuf = self.gallery.scale_thumbnail(self.image_list[self.gallery.list.index(item)].get_pixbuf().copy())
            self.model.append([pixbuf, basename, None])
            while gtk.events_pending():
                gtk.main_iteration(False)

    def change_user_account(self, obj, event=None):
        if obj == self.button5:
            self.token_key = None
            self.token_secret = None
        else:
            user = obj.get_active_text()
            if user:
                self.label3.set_text('')
                while gtk.events_pending():
                    gtk.main_iteration(False)
                if self.gallery.settings['dropbox_tokens'].has_key(user):
                    self.token_key = self.gallery.settings['dropbox_tokens'][user].get('key')
                    self.token_secret = self.gallery.settings['dropbox_tokens'][user].get('secret')
                else:
                    self.token_key = None
                    self.token_secret = None
        self.dropbox_login()

    def dropbox_login(self):
        if self.token_key and self.token_secret:
            self.get_token_cb(None, self.token_key, self.token_secret)
        else:
            if self.window1.get_property('visible'):
                    self.window1.iconify()
            self.wview_button.set_label(_('Continue'))
            self.browser.show_all()
            self.web_view.load_string(_('<div>Log into Dropbox to authorize GTKRawGallery and click OK.<p>Click Continue.</p></div>'), 'text/html', 'UTF-8', '#')
            while gtk.events_pending():
                gtk.main_iteration(False)

    def get_dropbox_session(self):
            try:
                self.session = dropbox.session.DropboxSession(_decode(dbx_data[0]), _decode(dbx_data[1]), "dropbox")
                self.request_token = self.session.obtain_request_token()
                url = self.session.build_authorize_url(self.request_token)
                self.web_view.load_uri(url)
            except dropbox.rest.ErrorResponse, e:
                gtk.gdk.beep()
                self.label3.set_text(_('Log-in failed!'))
                sys.stderr.write(e.error)
            except:
                gtk.gdk.beep()
                self.label3.set_text(_('Log-in failed!'))
                sys.stderr.write(traceback.format_exc())

    def get_token_cb(self, obj=None, t_key=None, t_secret=None):
        if obj == self.wview_button and obj.get_label() != 'gtk-ok':
            obj.set_label('gtk-ok')
            while gtk.events_pending():
                gtk.main_iteration(False)
            self.get_dropbox_session()
            return True
        if self.browser.get_property('visible'):
            self.browser.hide()
        self.window1.present()
        self.label3.set_text(_('authenticating...'))
        self.window1.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        while gtk.events_pending():
            gtk.main_iteration(False)
        try:
            if not self.session:
                self.session = dropbox.session.DropboxSession(_decode(dbx_data[0]), _decode(dbx_data[1]), "dropbox")
            if t_key:
                self.session.set_token(t_key, t_secret)
                access_token = self.session.token
            else:
                access_token = self.session.obtain_access_token(self.request_token)
            self.client = dropbox.client.DropboxClient(self.session)
            info = self.client.account_info()
            if info:
                self.label3.set_text(_('Retrieving directories...'))
                while gtk.events_pending():
                        gtk.main_iteration(False)
                metadata = self.client.metadata("/")
                dirs = []
                for item in metadata['contents']:
                    if item['is_dir']:
                        dirs.append(item['path'])
                self.populate_dir_box(dirs)
                self.button2.set_sensitive(True)
                self.window1.set_title(_('Upload into the DropBox of %s') % info['display_name'])
                self.label3.set_text(_('You are logged!'))
                self.gallery.settings['dropbox_user'] = info['display_name']
                self.gallery.settings['dropbox_tokens'][info['display_name']] = {'key':access_token.key, 'secret':access_token.secret}
                if self.gallery.settings.has_key('dropbox_tokens'):
                    self.combobox3.handler_block(self.id3)
                    for key in sorted(self.gallery.settings['dropbox_tokens'].keys()):
                        self.boxmodel3.append([key])
                    for row in self.boxmodel3:
                        if row[0] == info['display_name']:
                            self.combobox3.set_active_iter(row.iter)
                            break
                    else:
                        self.combobox3.set_active(0)
                    self.combobox3.handler_unblock(self.id3)
        except dropbox.rest.ErrorResponse, e:
            gtk.gdk.beep()
            self.label3.set_text(_('Log-in failed!'))
            sys.stderr.write(traceback.format_exc())
        self.window1.window.set_cursor(None)
                
    def main(self):
        self.progressbar1.hide()
        self.refresh_treeview()
        self.dropbox_login()
            
    def destroy(self, *obj):
        if self.uploading:
            self.stop = True
            return True
        self.window1.hide_all()
        self.gallery.unselect()
        while gtk.events_pending():
            gtk.main_iteration(False)
        self.model.clear()
        self.window1.destroy()
        self.browser.destroy()
        return True


