# -*- coding: utf-8 -*-
'''
GTKRawGallery Flickr publisher;
copyright © 2013 Daniele Isca
license: GNU GPL v.3 (see COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''
import gtk
import pygtk
import os.path
import os
import traceback
from subprocess import *
import webkit
import urlparse
import sys


class Publisher:
    def __init__(self, gallery, image_list, gtkrg_datadir):
        self.gallery = gallery
        self.tempfile = os.path.join(self.gallery.temp, 'tempfile.tiff')
        self.image_list = image_list
        self.flickr = None
        self.token = None
        self.frob = None
        self.stop = False
        self.auth_url = None
        self.uploading = False
        self.album = {}
        self.formats = ('.jpg', '.jpeg', '.gif', '.png', '.tiff', '.tif')
        wtree = gtk.Builder()
        wtree.add_from_file(os.path.join(gtkrg_datadir, 'publisher.ui'))
        self.initialize_widgets(wtree)
        self.boxmodel1 = gtk.ListStore(str)
        cell = gtk.CellRendererText()
        self.combobox1.set_model(self.boxmodel1)
        self.combobox1.pack_start(cell, True)
        self.combobox1.add_attribute(cell, 'text', 0)
        self.boxmodel3 = gtk.ListStore(str)
        cell = gtk.CellRendererText()
        self.combobox3.set_model(self.boxmodel3)
        self.combobox3.pack_start(cell, True)
        self.combobox3.add_attribute(cell, 'text', 0)
        self.window1.set_icon_from_file(self.gallery.get_icon_path('flickr.png'))
        self.make_flickr_treeview()
        self.window1.set_title(_('Publish on Flickr'))
        if self.gallery.window3.get_property('visible'):
            self.window1.set_transient_for(self.gallery.window3)
        elif self.gallery.window2.get_property('visible'):
            self.window1.set_transient_for(self.gallery.window2)
        else:
            self.window1.set_transient_for(self.gallery.window1)
        self.checkbutton1.set_property('visible', False)
        self.checkbutton2.set_property('visible', False)
        self.combobox2.set_property('visible', False)
        self.label5.set_property('visible', False)
        self.label6.set_text(_('Flickr user:'))
        # loads webview UI
        wview = gtk.Builder()
        wview.add_from_file(os.path.join(gtkrg_datadir, 'webview.ui'))
        self.browser = wview.get_object('window1')
        self.scrolled_window = wview.get_object('scrolledwindow1')
        self.wview_button = wview.get_object('button1')
        self.web_view = webkit.WebView()
        self.scrolled_window.add(self.web_view)
        self.connect_signals()
        self.browser.set_title(_('Log into Flickr'))
        self.browser.set_icon_from_file(self.gallery.get_icon_path('flickr.png'))
        self.browser.set_default_size(800, 600)
        self.browser.set_keep_above(True)
        if not self.gallery.settings.has_key('flickr_tokens'):
            self.gallery.settings['flickr_tokens'] = {}
        for key in sorted(self.gallery.settings['flickr_tokens'].keys()):
            self.boxmodel3.append([key])
        self.combobox3.handler_block(self.id3)
        user = self.gallery.settings.get('flickr_user')
        for row in self.boxmodel3:
            if row[0] == user:
                self.combobox3.set_active_iter(row.iter)
                break
        else:
            self.combobox3.set_active(0)
        self.combobox3.handler_unblock(self.id3)

    def load_started(self, *obj):
        if self.browser.window:
            self.browser.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))

    def load_finished(self, *obj):
        if self.browser.window:
            self.browser.window.set_cursor(None)

    def load_error(self, *obj):
        self.web_view.load_string(_('<div>Page Loading Error!</div>'), 'text/html', 'UTF-8', '#')
        
    def hide_webview(self, obj, event=None):
        self.browser.hide()
        return True

    def make_flickr_treeview(self):
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str, str, str, 'gboolean', 'gboolean', 'gboolean', 'gboolean', str)
        renderer_pix = gtk.CellRendererPixbuf()
        column_pix = gtk.TreeViewColumn("Preview", renderer_pix, pixbuf=0)
        column_pix.set_cell_data_func (renderer_pix, self.cell_data_function, None)
        self.treeview1.append_column(column_pix)
        cell = gtk.CellRendererText()                                                                           
        column_item1 = gtk.TreeViewColumn("Filename", cell, text=1)
        column_item1.set_cell_data_func (cell, self.cell_data_function, None)
        column_item1.set_resizable(True)                                                                                         
        self.treeview1.append_column(column_item1)
        cell = gtk.CellRendererText()
        cell.set_property("editable", True)
        cell.connect("edited", self.on_model_edited, self.model, 2)
        column_item2 = gtk.TreeViewColumn("Title", cell, text=2)
        column_item2.set_cell_data_func (cell, self.cell_data_function, None)
        column_item2.set_resizable(True)                                                                                         
        self.treeview1.append_column(column_item2)
        cell = gtk.CellRendererText()
        cell.set_property("editable", True)
        cell.connect("edited", self.on_model_edited, self.model, 3)
        column_item3 = gtk.TreeViewColumn("Caption", cell, text=3)
        column_item3.set_cell_data_func (cell, self.cell_data_function, None)
        column_item3.set_resizable(True)                                                                                         
        self.treeview1.append_column(column_item3)
        cell = gtk.CellRendererText()
        cell.set_property("editable", True)
        cell.connect("edited", self.on_model_edited, self.model, 4)
        column_item4 = gtk.TreeViewColumn("Tags", cell, text=4)
        column_item4.set_cell_data_func (cell, self.cell_data_function, None)
        column_item4.set_resizable(True)                                                                                         
        self.treeview1.append_column(column_item4)
        renderer_toggle1 = gtk.CellRendererToggle()
        renderer_toggle1.set_property('activatable', True)
        renderer_toggle1.connect('toggled', self.on_renderer_toggled, self.model, 5)
        column_toggle1 = gtk.TreeViewColumn("is public", renderer_toggle1)
        column_toggle1.set_cell_data_func (renderer_toggle1, self.cell_data_function, None)
        column_toggle1.set_attributes(renderer_toggle1, active=5)
        self.treeview1.append_column(column_toggle1)
        renderer_toggle2 = gtk.CellRendererToggle()
        renderer_toggle2.set_property('activatable', True)
        renderer_toggle2.connect('toggled', self.on_renderer_toggled, self.model, 6)
        column_toggle2 = gtk.TreeViewColumn("is friend", renderer_toggle2)
        column_toggle2.set_cell_data_func (renderer_toggle2, self.cell_data_function, None)
        column_toggle2.set_attributes(renderer_toggle2, active=6)
        self.treeview1.append_column(column_toggle2)
        renderer_toggle3 = gtk.CellRendererToggle()
        renderer_toggle3.set_property('activatable', True)
        renderer_toggle3.connect('toggled', self.on_renderer_toggled, self.model, 7)
        column_toggle3 = gtk.TreeViewColumn("is family", renderer_toggle3)
        column_toggle3.set_cell_data_func (renderer_toggle3, self.cell_data_function, None)
        column_toggle3.set_attributes(renderer_toggle3, active=7)
        self.treeview1.append_column(column_toggle3)
        renderer_toggle4 = gtk.CellRendererToggle()
        renderer_toggle4.set_property('activatable', True)
        renderer_toggle4.connect('toggled', self.on_renderer_toggled, self.model, 8)
        column_toggle4 = gtk.TreeViewColumn("hidden", renderer_toggle4)
        column_toggle4.set_cell_data_func (renderer_toggle4, self.cell_data_function, None)
        column_toggle4.set_attributes(renderer_toggle4, active=8)
        self.treeview1.append_column(column_toggle4)
        cell = gtk.CellRendererText()
        column_status = gtk.TreeViewColumn("Status", cell, text=9)
        column_status.set_cell_data_func (cell, self.cell_data_function, None)
        self.treeview1.append_column(column_status)
        self.treeview1.set_model(self.model)

    def cell_data_function(self, column, cell, model, _iter, data=None):
        value = model.get_value(_iter, 9)
        if value == None:
            color = self.treeview1.rc_get_style().base[gtk.STATE_NORMAL].to_string()
        elif value == 'uploading...' or value == 'converting...':
            color = '#CD7300'
        elif value == 'error!':
            color = 'red'
        else:
            color = self.treeview1.rc_get_style().base[gtk.STATE_NORMAL].to_string()
        cell.set_property('cell_background', color)

    def on_model_edited(self, cell, path, new_text, model, column):
        if not new_text.isspace():
            model.set_value(model.get_iter(path), column, new_text)

    def on_renderer_toggled(self, cell, path, model, position):
        model[path][position] = not model[path][position]
        
    def initialize_widgets(self, tree):
        for widget in tree.get_objects():
            name = gtk.Buildable.get_name(widget)
            if name and not hasattr(self, name):
                setattr(self, name, widget)

    def connect_signals(self):
        self.button1.connect('clicked', self.destroy)
        self.button2.connect('clicked', self.upload)
        self.window1.connect('delete_event', self.destroy)
        self.window1.connect('key_press_event', self.on_key_press_event)
        self.radiobutton1.connect('toggled', self.on_radiobutton_toggled)
        self.button5.connect('clicked', self.change_user_account)
        self.id3 = self.combobox3.connect('changed', self.change_user_account)
        self.web_view.connect('load_error', self.load_error)
        self.web_view.connect('load_finished', self.load_finished)
        self.web_view.connect('load_started', self.load_started)
        self.browser.connect('delete_event', self.hide_webview)
        self.wview_button.connect('clicked', self.get_token_cb)

    def on_radiobutton_toggled(self, obj):
        self.label4.set_sensitive(not obj.get_active())
        self.entry4.set_sensitive(not obj.get_active())
        self.entry1.set_sensitive(not obj.get_active())
        self.combobox1.set_sensitive(obj.get_active())

    def on_key_press_event(self, obj, event):
        keyname = gtk.gdk.keyval_name(event.keyval)
        if keyname == 'Delete' and len(self.model):
            selection = self.treeview1.get_selection()
            model, iter_ = selection.get_selected()
            if iter_:
                model.remove(iter_)

    def populate_album_box(self, albums, text=None):
        self.boxmodel1.clear()
        for album in sorted(albums):
            self.boxmodel1.append([album])
        if text:
            for row in self.boxmodel1:
                if row == text:
                    self.combobox1.set_active_iter(row.iter)
                    break
        else:
            self.combobox1.set_active(0)

    def get_album_name(self):
        description = None
        create = False
        if self.radiobutton1.get_active():
            album = self.combobox1.get_active_text() or None
        else:
            album = self.entry1.get_text() or None
            if album.isspace():
                album = None
            description = self.entry4.get_text() or None
            if album:
                create = True
        return album, description, create

    def get_source_file(self, basename, iter_):
        source = self.gallery.get_real_path(self.rawpath[basename])
        if self.gallery.window2.get_property('visible'):
            pixbuf = self.gallery.get_print_pixbuf(True)
            if self.checkbutton3.get_active():
                photo = self.scale_image(pixbuf)
            else:
                filename = os.path.join(self.gallery.temp, 'mytemp.jpeg')
                pixbuf.save(filename, "jpeg", {"quality":"100"})
                photo = filename
        else:
            if basename.lower().endswith(self.formats):
                filename = source
                if self.checkbutton3.get_active():
                    filename = self.scale_image(filename)
            elif basename.lower().endswith(self.gallery.raw_formats):
                self.model.set_value(iter_, 9, 'converting...')
                while gtk.events_pending():
                    gtk.main_iteration(False)
                filename = self.process_raw(source)
                if self.checkbutton3.get_active():
                    filename = self.scale_image(filename)
            else:
                pixbuf = gtk.gdk.pixbuf_new_from_file(source)
                if self.checkbutton3.get_active():
                    width, height = pixbuf.get_width(), pixbuf.get_height()
                    if width > 720 or height > 720:
                        ratio = float(width) / height
                        if ratio >= 1:
                            pixbuf = pixbuf.scale_simple(720, int(height * float(720 / width)), gtk.gdk.INTERP_BILINEAR)
                        else:
                            pixbuf = pixbuf.scale_simple(int(width * float(720 / height)), 720, gtk.gdk.INTERP_BILINEAR)
                tempfile = os.path.join(self.gallery.temp, 'mytemp.jpeg')
                pixbuf.save(tempfile, "jpeg", {"quality":"100"})
                filename = tempfile
        return filename

    def process_raw(self, filename):
        format_ = os.path.splitext(filename)[1]
        fname = os.path.basename(filename)[:-len(format_)]
        outfile = self.tempfile
        options = self.gallery.dcraw_options2()
        if '-h' not in options:
            options.insert(1, '-h')
        command = ' '.join(options)
        p = Popen(command % (filename, outfile), shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, universal_newlines=True, env={'LANG':'en_Us'}, creationflags = self.gallery.CREATIONFLAGS)
        output = p.communicate()[0]
        p.stdout.close()
        p.stdin.close()
        return outfile

    def scale_image(self, photo):
        if isinstance(photo, str):
            pixbuf = gtk.gdk.pixbuf_new_from_file(photo)
        else:
            pixbuf = photo
        width, height = pixbuf.get_width(), pixbuf.get_height()
        if width > 720 or height > 720:
            ratio = float(width) / height
            if ratio >= 1:
                pixbuf = pixbuf.scale_simple(720, int(height * float(720) / width), gtk.gdk.INTERP_BILINEAR)
            else:
                pixbuf = pixbuf.scale_simple(int(width * float(720) / height), 720, gtk.gdk.INTERP_BILINEAR)
            filename = os.path.join(self.gallery.temp, 'mytemp.jpeg')
            pixbuf.save(filename, "jpeg", {"quality":"100"})
            photo = filename
        return photo

    def upload(self, obj=None):
        if len(self.model):
            resp= self.flickr.test_login()
            if resp.attrib['stat'] == 'ok':
                self.button2.set_sensitive(False)
                self.stop = False
                self.uploading = True
                self.upload_to_flickr()
                self.stop = False
                self.uploading = False
                self.button2.set_sensitive(True)
            else:
                self.label3.set_text(_('connection error!'))
            
    def upload_to_flickr(self):
        for row in self.model:
            self.model.set_value(row.iter, 9, None)
        album, album_description, create = self.get_album_name()
        album_id = None
        d = {True:"1", False:"0"}
        d2 = {True:"2",False:"1"}
        def get_tags(text):
            if text == '<edit>':
                return None
            tags = ''
            keywords = text.split(',')
            for item in keywords:
                if ' ' in item:
                    tags += '"%s"'
                else:
                    tags += '%s'
                tags += ' '
            tags.rstrip()
            if tags:
                return tags % tuple(keywords)
            return None
        self.fraction = 1.0 / len(self.model)
        self.progressbar1.set_fraction(0)
        self.progressbar1.show()
        while gtk.events_pending():
            gtk.main_iteration(False)
        def callback(progress, done):
            if done:
                print 'upload completed'
            else:
                print 'At %s ' % progress
                self.progressbar1.set_fraction(self.progressbar1.get_fraction() + float(progress) * self.fraction / 10.0)
        for row in self.model:
            if self.stop:
                break
            image = self.get_source_file(row[1], row.iter)
            title = row[2] or None
            if title == '<edit>':
                title = None
            description = row[3] or None
            if description == '<edit>':
                description = None
            tags = get_tags(row[4])
            public = d[row[5]]
            friend = d[row[6]]
            family = d[row[7]]
            hidden = d2[row[8]]
            self.progressbar1.set_text(_('Uploading %s ...') % row[1])
            self.model.set_value(row.iter, 9, 'uploading...')
            while gtk.events_pending():
                gtk.main_iteration(False)
            for item in range(3):
                resp = self.flickr.upload(filename=image, callback=callback, title=title, description=description, tags=tags, is_public=public, is_friend=friend, is_family=family, hidden=hidden)
                if resp.attrib['stat'] == 'ok':
                    break
            photo_id = None
            if resp.attrib['stat'] == 'ok':
                id_ = resp.find('photoid')
                if id_ is not None:
                    photo_id = id_.text
                    self.model.set_value(row.iter, 9, 'success!')
                else:
                    self.model.set_value(row.iter, 9, 'error!')
            self.progressbar1.set_fraction(self.progressbar1.get_fraction() + self.fraction)
            self.treeview1.scroll_to_cell(self.model.get_path(row.iter))
            while gtk.events_pending():
                gtk.main_iteration(False)
            if album:
                if create and photo_id:
                    album_id = self.create_new_album(album, photo_id, album_description)
                    create = False
                    if album_id:
                        self.album[album.decode('utf-8')] = album_id
                        self.populate_album_box(self.album.keys(),self.combobox1.get_active_text())
                else:
                    if photo_id:
                        if album_id:
                            self.add_to_album(album_id, photo_id)
                        else:    
                            self.add_to_album(self.album[album.decode('utf-8')], photo_id)
        if os.path.isfile(self.tempfile):
            os.remove(self.tempfile)
        self.progressbar1.hide()

    def create_new_album(self, album_id, photo_id, description):
        resp = self.flickr.photosets_create(title=album_id, description=description, primary_photo_id=photo_id)
        if resp.attrib['stat'] == 'ok':
            return resp.find('photoset').attrib['id']
        return None

    def add_to_album(self, album_id, photo_id):
        resp = self.flickr.photosets_addPhoto(photoset_id=album_id, photo_id=photo_id, format='json')
        print resp
                         
    def refresh_treeview(self):
        self.rawpath = {}
        self.model.clear()
        for item in self.gallery.selected:
            basename = os.path.basename(self.gallery.get_real_path(item))
            self.rawpath[basename] = item
            if self.image_list[0] == self.gallery.image28:
                pixbuf = self.gallery.scale_thumbnail(self.image_list[0].get_pixbuf().copy())
            else:
                pixbuf = self.gallery.scale_thumbnail(self.image_list[self.gallery.list.index(item)].get_pixbuf().copy())
            self.model.append([pixbuf, basename, '<edit>','<edit>','<edit>', True, False, False, False, None])
            while gtk.events_pending():
                gtk.main_iteration(False)

    def change_user_account(self, obj):
        token = None
        if obj == self.button5:
            self.label3.set_text('')
            if self.window1.get_property('visible'):
                self.window1.iconify()
                while gtk.events_pending():
                    gtk.main_iteration(False)
        else:
            user = self.combobox3.get_active_text()
            if user:
                token = self.gallery.settings['flickr_tokens'].get(user)
        if self.flickr_login(token):
            self.window1.present()
            self.progressbar1.hide()
            self.refresh_flickr_albums()
        
    def refresh_flickr_albums(self, username=None):
        if self.window1.window:
            self.window1.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        try:
            resp= self.flickr.test_login()
            if resp.attrib['stat'] == 'ok':
                self.label3.set_text(_('Retrieving albums...'))
                while gtk.events_pending():
                        gtk.main_iteration(False)
                sets = self.flickr.photosets_getList(user_id=None)
                if sets.attrib['stat'] == 'ok':
                    photosets = sets.find('photosets').findall('photoset')
                    album_list = []
                    if photosets:
                        for item in photosets:
                            title = item.find('title').text
                            self.album[title] = item.attrib['id']
                            album_list.append(title)
                        self.populate_album_box(album_list)
                    else:
                        self.label3.set_text('')
                    self.label3.set_text(_('you have %s albums') % len(album_list))
                self.button2.set_sensitive(True)
            else:
                self.label3.set_text(_('Login failed!'))
            self.window1.window.set_cursor(None)
            return True
        except:
            self.label3.set_text(_('Login failed!'))
            if self.window1.window:
                self.window1.window.set_cursor(None)
            return True

    def flickr_auth(self, frob, perms):
            self.auth_url = self.flickr.auth_url(perms, frob)
            self.wview_button.set_label(_('Continue'))
            self.web_view.load_string(_('<div>Log into Flickr to authorize GTKRawGallery and click OK.<p>Click on continue.</p></div>'), 'text/html', 'UTF-8', '#')
            self.browser.show_all()
            while gtk.events_pending():
                gtk.main_iteration(False)
            
    def flickr_login(self, token=None):
        self.gallery.window1.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        while gtk.events_pending():
                gtk.main_iteration(False)
        import flickrapi
        from gtkrg_profiles import fli_data
        def decode_(data):
            return ''.join([chr(i) for i in data])
        self.flickr = flickrapi.FlickrAPI(api_key=decode_(fli_data[0]), secret=decode_(fli_data[1]), token=token, store_token=False)
        try:
            self.token = None
            (self.token, self.frob) = self.flickr.get_token_part_one(perms='write', auth_callback=self.flickr_auth)
            if self.token:
                resp= self.flickr.test_login()
                if resp.attrib['stat'] == 'ok':
                    username = resp[0].find('username').text
                    self.window1.set_title(_('Publish on %s Flickr account') % username)
                    self.gallery.settings['flickr_user'] = username
                    self.gallery.settings['flickr_tokens'][username] = self.token
                    self.gallery.window1.window.set_cursor(None)
                    return True
                else:
                    gtk.gdk.beep()
                    self.label3.set_text(_('Log-in failed!'))
                    self.gallery.window1.window.set_cursor(None)
                    return None
            else:
                    self.gallery.window1.window.set_cursor(None)
                    return None
        except:
            gtk.gdk.beep()
            self.label3.set_text(_('Log-in failed!'))
            self.gallery.window1.window.set_cursor(None)
            return None

    def get_token_cb(self, obj):
        if self.wview_button.get_label() == 'gtk-ok':
            self.browser.hide()
            self.progressbar1.hide()
            self.window1.present()
            self.gallery.window1.window.set_cursor(None)
            while gtk.events_pending():
                gtk.main_iteration(False)
            token = self.flickr.get_token_part_two((self.token, self.frob))
            resp = self.flickr.test_login()
            if resp.attrib['stat'] == 'ok':
                username = resp[0].find('username').text
                self.window1.set_title(_('Publish on %s Flickr account') % username)
                self.gallery.settings['flickr_user'] = username
                self.gallery.settings['flickr_tokens'][username] = token
                self.combobox3.handler_block(self.id3)
                self.boxmodel3.clear()
                for key in sorted(self.gallery.settings['flickr_tokens'].keys()):
                    self.boxmodel3.append([key])
                for row in self.boxmodel3:
                    if row[0] == username:
                        self.combobox3.set_active_iter(row.iter)
                        break
                else:
                    self.combobox3.set_active(0)
                self.combobox3.handler_unblock(self.id3)
                while gtk.events_pending():
                    gtk.main_iteration(False)
                self.refresh_treeview()
                self.refresh_flickr_albums()
            else:
                gtk.gdk.beep()
                self.label3.set_text(_('Log-in failed!'))
                self.progressbar1.hide()
        else:
            if self.auth_url:
                self.web_view.load_uri(self.auth_url)
                self.wview_button.set_label('gtk-ok')
                       
    def main(self):
        self.gallery.window2.set_modal(False)
        token = None
        user = self.gallery.settings.get('flickr_user')
        if user and self.gallery.settings.get('flickr_tokens'):
            token = self.gallery.settings['flickr_tokens'].get(user)
        if self.flickr_login(token):
            self.window1.show()
            self.progressbar1.hide()
            while gtk.events_pending():
                gtk.main_iteration(False)
            self.refresh_treeview()
            self.refresh_flickr_albums()

    def destroy(self, *obj):
        self.gallery.window2.set_modal(True)
        if self.uploading:
            self.stop = True
            return True
        self.window1.hide_all()
        self.gallery.unselect()
        while gtk.events_pending():
            gtk.main_iteration(False)
        if self.flickr:
            del self.flickr
        self.model.clear()
        self.window1.destroy()
        self.browser.destroy()


