# -*- coding: cp1252 -*-
'''
GTKRawGallery print module;
copyright  2013 Daniele Isca
license: GNU GPL v.3 (see COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''

import gtk
import pygtk
import os.path
import warnings
import sys


class Printer(object):
        def __init__(self, gallery, gtkrg_datadir):
                self.gallery = gallery
                self.gtkrg_datadir = gtkrg_datadir
                self.p_pixbuf = None
                self.print_settings = self.gallery.print_settings
                self.page_setup = self.gallery.page_setup
                self.print_operation = gtk.PrintOperation()
                self.print_operation.set_unit(gtk.UNIT_PIXEL)
                if self.print_settings: 
                        self.print_operation.set_print_settings(self.print_settings)
                else:
                        self.print_settings = gtk.PrintSettings()
                self.print_settings.set_scale(100)
                if not self.page_setup:
                        self.page_setup = gtk.PageSetup()
                        self.page_setup.set_orientation(gtk.PAGE_ORIENTATION_PORTRAIT)
                        paper_size = gtk.PaperSize(gtk.PAPER_NAME_A4)
                        self.page_setup.set_paper_size_and_default_margins(paper_size)
                self.print_operation.set_default_page_setup(self.page_setup)
                self.print_operation.set_n_pages(1)
                self.print_operation.set_print_settings(self.print_settings)
                self.print_operation.set_custom_tab_label('Settings')
                self.print_operation.connect('create_custom_widget', self.create_custom_widget)
                self.print_operation.connect('begin_print', self.begin_print)
                self.draw_page_id = self.print_operation.connect('draw_page', self.draw_page)
                self.gallery.window2.set_modal(False)
                if self.gallery.window2.get_property('visible'):
                        parent = self.gallery.window2
                elif self.gallery.window3.get_property('visible'):
                        parent = self.gallery.window3
                else:
                        parent = self.gallery.window1
                res = self.print_operation.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG, parent)
                if res == gtk.PRINT_OPERATION_RESULT_APPLY:
                        self.print_settings = self.print_operation.get_print_settings()
                self.gallery.window2.set_modal(True)
                        
        def on_preview_exposed(self, area, event):
                scale = self.print_settings.get_scale() / 100
                self.print_operation.handler_block(self.draw_page_id)                    
                cr = area.window.cairo_create()
                paper_w = self.page_setup.get_paper_width(gtk.UNIT_MM)
                paper_h = self.page_setup.get_paper_height(gtk.UNIT_MM)
                left_margin = self.page_setup.get_left_margin(gtk.UNIT_MM)
                right_margin = self.page_setup.get_right_margin(gtk.UNIT_MM)
                top_margin = self.page_setup.get_top_margin(gtk.UNIT_MM)
                bottom_margin = self.page_setup.get_bottom_margin(gtk.UNIT_MM)
                ratio = float(paper_h) / paper_w
                if ratio == 1:
                        cr_w = cr_h = 100
                elif ratio < 1:
                        cr_w = 100
                        cr_h = int(100* ratio)
                else:
                        cr_h = 150
                        cr_w = int(cr_h / ratio)
                        if cr_w > 150:
                                cr_w = 100
                                cr_h = int(100 * ratio)
                self.p_drawingarea.set_size_request(cr_w + 2,cr_h + 2)
                cr.set_line_width(1)
                cr.rectangle(0.5, 0.5, cr_w, cr_h)
                cr.stroke_preserve()
                cr.set_source_rgb(1, 1, 1)
                cr.fill()
                while gtk.events_pending():
                        gtk.main_iteration(False)
                page_w, page_h = self.page_setup.get_page_width(gtk.UNIT_MM), self.page_setup.get_page_height(gtk.UNIT_MM)
                if not self.p_pixbuf:
                        if self.gallery.list[self.gallery.indice].lower().endswith(self.gallery.raw_formats):
                                if self.gallery.window2.get_property('visible') and self.gallery.raw_preview:
                                        if self.gallery.expander17.get_expanded():
                                                self.p_pixbuf = self.gallery.current_pixbuf.copy()
                                        else:
                                                self.p_pixbuf = self.gallery.image28.get_pixbuf()
                                        if self.gallery.count == 3:
                                                self.p_pixbuf = self.p_pixbuf.rotate_simple(self.gallery.gradi[1])
                                        elif self.gallery.count == 1:
                                                self.p_pixbuf = self.p_pixbuf.rotate_simple(self.gallery.gradi[3])
                                        elif self.gallery.count == 2:
                                                self.p_pixbuf = self.p_pixbuf.rotate_simple(self.gallery.gradi[2])
                                else:        
                                        options = self.gallery.dcraw_options2()
                                        self.gallery.extract_fullsize_image(self.gallery.get_real_path(self.gallery.list[self.gallery.indice]), options, True)
                                        self.gallery.full_process.join()
                                        self.p_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.gallery.unistr(os.path.join(self.gallery.temp, 'fullsize_image.tiff')), 210, 210)
                        else:
                                if self.gallery.window2.get_property('visible'):
                                        if self.gallery.expander17.get_expanded():
                                                self.p_pixbuf = self.gallery.current_pixbuf.copy()
                                        else:
                                                self.p_pixbuf = self.gallery.image28.get_pixbuf()
                                else:
                                        self.p_pixbuf = self.gallery.swin2_image.get_pixbuf()
                                if self.gallery.count == 3:
                                        self.p_pixbuf = self.p_pixbuf.rotate_simple(self.gallery.gradi[1])
                                elif self.gallery.count == 1:
                                        self.p_pixbuf = self.p_pixbuf.rotate_simple(self.gallery.gradi[3])
                                elif self.gallery.count == 2:
                                        self.p_pixbuf = self.p_pixbuf.rotate_simple(self.gallery.gradi[2])
                if self.p_pixbuf:
                        pixbuf = self.p_pixbuf.copy()
                else:
                        return True
                width, height = pixbuf.get_width(), pixbuf.get_height()
                ratio = float(width) / height
                cr_page = max(50, cr_w - int(left_margin * cr_w/paper_w ) - int(right_margin * cr_w/paper_w )), max(50, cr_h - int(top_margin * cr_w/paper_w) - int(bottom_margin * cr_w/paper_w))
                if ratio > 1:
                        width, height = cr_page[0], cr_page[0] / ratio
                else:
                        width, height = cr_page[0], cr_page[0] / ratio
                if height > cr_page[1]:
                        width, height = cr_page[1] * ratio , cr_page[1]
                pixbuf = pixbuf.scale_simple(int(width * scale), int(height * scale), gtk.gdk.INTERP_BILINEAR)
                if self.gallery.checkbutton41.get_active() and not self.gallery.window2.get_property('visible'):
                        pixbuf = self.gallery.profile_pixbuf(pixbuf)
                cr.set_source_pixbuf(pixbuf, int(left_margin * cr_w/paper_w ), int(top_margin * cr_h/paper_h))
                cr.paint()
                self.print_operation.handler_unblock(self.draw_page_id)
                        
        def create_custom_widget(self, operation):
                tree = gtk.Builder()
                tree.set_translation_domain('gtkrawgallery')
                tree.add_from_file(os.path.join(self.gtkrg_datadir, 'print.ui'))
                self.p_hbox1 = tree.get_object('hbox1')
                if sys.platform.startswith('win') and self.gallery.radiobutton13.get_active():
                        self.p_hbox1.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#ece9f8'))
                self.p_spinbutton1 = tree.get_object('spinbutton1')
                self.p_spinbutton2 = tree.get_object('spinbutton2')
                self.p_spinbutton3 = tree.get_object('spinbutton3')
                self.p_spinbutton4 = tree.get_object('spinbutton4')
                self.p_spinbutton5 = tree.get_object('spinbutton5')
                self.p_spinbutton6 = tree.get_object('spinbutton6')
                self.p_spinbutton7 = tree.get_object('spinbutton7')
                self.p_drawingarea = tree.get_object('drawingarea1')
                self.p_checkbutton2 = tree.get_object('checkbutton2')
                self.p_entry1 = tree.get_object('entry1')
                self.p_entry2 = tree.get_object('entry2')
                self.p_label3 = tree.get_object('label3')
                self.p_combobox2 = tree.get_object('combobox2')
                self.p_combobox1 = tree.get_object('combobox1')
                self.p_radiobutton1 = tree.get_object('radiobutton1')
                self.p_radiobutton2 = tree.get_object('radiobutton2')
                hbox4 = tree.get_object('hbox4')
                hbox4.show()
                self.print_settings.set_scale(100)
                self.p_spinbutton5.set_value(100)
                self.p_spinbutton5.connect('button-release-event', self.on_p_spinbutton5_value_changed)
                self.p_drawingarea.connect('expose_event', self.on_preview_exposed)
                self.p_checkbutton2.connect('toggled', self.on_custom_size_changed)
                self.p_drawingarea.set_size_request(210,210)
                self.p_papers = {'A3' : [gtk.PAPER_NAME_A3, 11.69, 16.54],
                                 'A4' : [gtk.PAPER_NAME_A4, 8.27, 11.69],
                                 'A5' : [gtk.PAPER_NAME_A5, 5.83, 8.27],
                                 'US Letter' : [gtk.PAPER_NAME_LETTER, 8.5, 11],
                                 'US Legal' : [gtk.PAPER_NAME_EXECUTIVE, 8.5, 14],
                                 'B5' : [gtk.PAPER_NAME_B5, 6.93, 9.84],
                                 'ROC 16K' : [7.75, 10.75],
                                 'JB5' : [7.17, 10.12],
                                 '#10 Envelope' : [4.13, 9.5],
                                 'DL Envelope' : [4.33, 8.66],
                                 'Choukei 3 Envelope' : [4.72, 9.25],
                                 'Tabloid' : [11, 17]}
                cell = gtk.CellRendererText()
                boxmodel2 = gtk.ListStore(str)
                boxmodel2.append(['millimeters'])
                boxmodel2.append(['inches'])
                self.p_combobox2.pack_start(cell, True)
                self.p_combobox2.add_attribute(cell, 'text', 0)
                self.p_combobox2.set_model(boxmodel2)
                self.p_combobox2.set_active(0)
                self.p_combobox2.connect('changed', self.on_p_combobox2_changed)
                self.p_combobox1.connect('changed', self.on_p_combobox1_changed)
                boxmodel1 = gtk.ListStore(str)
                for key in sorted(self.p_papers.keys()):
                        boxmodel1.append([key])
                self.p_combobox1.pack_start(cell, True)
                self.p_combobox1.add_attribute(cell, 'text', 0)
                self.p_combobox1.set_model(boxmodel1)
                self.p_combobox1.set_active(2)
                self.p_spinbutton1.connect('button_release_event', self.set_margins, self.page_setup.set_left_margin)
                self.p_spinbutton2.connect('button_release_event', self.set_margins, self.page_setup.set_right_margin)
                self.p_spinbutton3.connect('button_release_event', self.set_margins, self.page_setup.set_top_margin)
                self.p_spinbutton4.connect('button_release_event', self.set_margins, self.page_setup.set_bottom_margin)
                self.p_spinbutton6.connect('value-changed', self.on_p_spinbutton6_value_changed)
                self.p_spinbutton7.connect('value-changed', self.on_p_spinbutton6_value_changed)
                self.p_radiobutton2.connect('toggled', self.on_p_radiobutton2_toggled)
                self.p_spinbutton1.set_value(self.page_setup.get_left_margin(gtk.UNIT_MM))
                self.p_spinbutton2.set_value(self.page_setup.get_right_margin(gtk.UNIT_MM))
                self.p_spinbutton3.set_value(self.page_setup.get_top_margin(gtk.UNIT_MM))
                self.p_spinbutton4.set_value(self.page_setup.get_bottom_margin(gtk.UNIT_MM))
                if self.p_combobox2.get_active():
                        step = 0.1
                else:
                        step = 1.0
                self.p_spinbutton6.set_increments(step,0)
                self.p_spinbutton7.set_increments(step,0)
                self.p_spinbutton6.set_range(10, 1000)
                self.p_spinbutton7.set_range(10, 1000)
                self.p_spinbutton6.set_value(self.page_setup.get_paper_width(gtk.UNIT_MM))
                self.p_spinbutton7.set_value(self.page_setup.get_paper_height(gtk.UNIT_MM))
                if self.p_radiobutton1.get_active():
                        self.page_setup.set_orientation(gtk.PAGE_ORIENTATION_PORTRAIT)
                else:
                        self.page_setup.set_orientation(gtk.PAGE_ORIENTATION_LANDSCAPE)
                self.p_label3.set_text(_('%(w).1f x %(h).1f millimeters') % {'w':self.page_setup.get_paper_width(gtk.UNIT_MM), 'h':self.page_setup.get_paper_height(gtk.UNIT_MM)})
                self.p_hbox1.unparent()
                return self.p_hbox1

        def on_p_radiobutton2_toggled(self, obj):
                left_margin = self.page_setup.get_left_margin(gtk.UNIT_MM)
                right_margin = self.page_setup.get_right_margin(gtk.UNIT_MM)
                top_margin = self.page_setup.get_top_margin(gtk.UNIT_MM)
                bottom_margin = self.page_setup.get_bottom_margin(gtk.UNIT_MM)
                if obj.get_active():
                        self.page_setup.set_orientation(gtk.PAGE_ORIENTATION_LANDSCAPE)
                        self.page_setup.set_left_margin(bottom_margin, gtk.UNIT_MM)
                        self.page_setup.set_right_margin(top_margin, gtk.UNIT_MM)
                        self.page_setup.set_top_margin(left_margin, gtk.UNIT_MM)
                        self.page_setup.set_bottom_margin(right_margin, gtk.UNIT_MM)
                else:
                        self.page_setup.set_orientation(gtk.PAGE_ORIENTATION_PORTRAIT)
                        self.page_setup.set_left_margin(top_margin, gtk.UNIT_MM)
                        self.page_setup.set_right_margin(bottom_margin, gtk.UNIT_MM)
                        self.page_setup.set_top_margin(right_margin, gtk.UNIT_MM)
                        self.page_setup.set_bottom_margin(left_margin, gtk.UNIT_MM)
                self.on_p_combobox2_changed(self.p_combobox2)
                self.p_drawingarea.queue_draw()

        def on_p_spinbutton6_value_changed(self, obj):
                if self.p_checkbutton2.get_active():
                        if self.p_combobox2.get_active():
                                unit = gtk.UNIT_INCH
                        else:
                                unit = gtk.UNIT_MM
                        paper_size = gtk.paper_size_new_custom('custom', 'custom', self.p_spinbutton6.get_value(), self.p_spinbutton7.get_value(), unit)
                        self.page_setup.set_paper_size(paper_size)
                        self.p_drawingarea.queue_draw()        

        def on_p_spinbutton5_value_changed(self, obj, event):
                self.print_settings.set_scale(obj.get_value())
                self.print_operation.set_print_settings(self.print_settings)
                self.p_drawingarea.queue_draw()

        def begin_print(self, operation, context):
                if self.p_combobox2.get_active():
                        unit = gtk.UNIT_INCH
                else:
                        unit = gtk.UNIT_MM
                self.page_setup.set_left_margin(self.p_spinbutton1.get_value(), unit)
                self.page_setup.set_right_margin(self.p_spinbutton2.get_value(), unit)
                self.page_setup.set_top_margin(self.p_spinbutton3.get_value(), unit)
                self.page_setup.set_bottom_margin(self.p_spinbutton4.get_value(), unit)
                self.print_settings.set_quality(gtk.PRINT_QUALITY_HIGH)
                operation.set_default_page_setup(self.page_setup)
                operation.set_print_settings(self.print_settings)

        def set_margins(self, obj, event, fun):
                if self.p_combobox2.get_active():
                        unit = gtk.UNIT_INCH
                else:
                        unit = gtk.UNIT_MM
                fun(obj.get_value(), unit)
                self.print_operation.set_default_page_setup(self.page_setup)
                self.p_drawingarea.queue_draw()
                 
        def draw_page(self, operation, context, page_nr):
                cr = context.get_cairo_context()
                scale = self.print_settings.get_scale() / 100
                with warnings.catch_warnings():
                        warnings.simplefilter("ignore")
                        pixbuf = self.gallery.get_print_pixbuf()
                        r = float(pixbuf.get_width()) / pixbuf.get_height()
                        if self.p_radiobutton2.get_active() and r <= 1:
                                h = context.get_height() * scale
                                w = int(h * r)
                        else:
                                w = context.get_width() * scale
                                h = int(w / r)
                        pixbuf = pixbuf.scale_simple(int(w), int(h), gtk.gdk.INTERP_BILINEAR)
                cr.set_source_pixbuf(pixbuf, 0, 0)
                cr.paint()

        def on_p_combobox1_changed(self, obj):
                key = obj.get_active_text()
                paper = self.p_papers[key]
                if len(paper) == 3:
                        paper_size = gtk.PaperSize(paper[0])
                        self.page_setup.set_paper_size_and_default_margins(paper_size)
                else:
                        paper_size = gtk.paper_size_new_custom(key, key, paper[-2], paper[-1], gtk.UNIT_INCH)
                        self.page_setup.set_paper_size(paper_size)
                        self.page_setup.set_left_margin(0.25, gtk.UNIT_INCH)
                        self.page_setup.set_right_margin(0.25, gtk.UNIT_INCH)
                        self.page_setup.set_top_margin(0.25, gtk.UNIT_INCH)
                        self.page_setup.set_bottom_margin(0.25, gtk.UNIT_INCH)
                self.on_p_combobox2_changed(self.p_combobox2)
                self.p_drawingarea.queue_draw()

        def on_custom_size_changed(self, obj):
                self.on_p_combobox1_changed(self.p_combobox1)
               
        def on_p_combobox2_changed(self, obj):
                if obj.get_active():
                        unit = gtk.UNIT_INCH
                        step = 0.1
                        digits = 1
                        lower = 0.2
                else:
                        unit = gtk.UNIT_MM
                        step = 1
                        digits = 0
                        lower = 10
                self.p_spinbutton1.set_increments(step,0)
                self.p_spinbutton2.set_increments(step,0)
                self.p_spinbutton3.set_increments(step,0)
                self.p_spinbutton4.set_increments(step,0)
                self.p_label3.set_text(_('%(w).3f x %(h).3f %(text)s') % {'w':self.page_setup.get_paper_width(unit), 'h':self.page_setup.get_paper_height(unit), 'text':self.p_combobox2.get_active_text()})
                self.p_spinbutton1.set_value(self.page_setup.get_left_margin(unit))
                self.p_spinbutton2.set_value(self.page_setup.get_right_margin(unit))
                self.p_spinbutton3.set_value(self.page_setup.get_top_margin(unit))
                self.p_spinbutton4.set_value(self.page_setup.get_bottom_margin(unit))
                self.p_spinbutton6.set_digits(digits)
                self.p_spinbutton7.set_digits(digits)
                self.p_spinbutton6.set_increments(step,0)
                self.p_spinbutton7.set_increments(step,0)
                self.p_spinbutton6.set_range(lower, 1000)
                self.p_spinbutton7.set_range(lower, 1000)
                self.p_spinbutton6.set_value(self.page_setup.get_paper_width(unit))
                self.p_spinbutton7.set_value(self.page_setup.get_paper_height(unit))

                
