# -*- coding: cp1252 -*-
'''
GTKRawGallery smtp client module;
copyright  2013 Daniele Isca
license: GNU GPL v.3 (see COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''

import smtplib, os, gobject
from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.Utils import formatdate
from email import Encoders
from sys import stderr

 
class ConnectionError(smtplib.SMTPException): pass
class LoginError(smtplib.SMTPException): pass
class DisconnectionError(smtplib.SMTPException): pass
class EmailSendError(smtplib.SMTPException): pass

class MailServer(smtplib.SMTP):
        def __init__(self, host, port, pbar):
                self.pbar = pbar
                smtplib.SMTP.__init__(self, host, port)

        def split_msg(self, msg):
                n = 0
                size = len(msg) / 100
                gobject.idle_add(self.pbar.set_fraction, 0)
                for item in range(99):
                        self.send(msg[n:n+size])
                        gobject.idle_add(self.pbar.set_fraction, self.pbar.get_fraction() + 0.0099)
                        n += size
                self.send(msg[n:])
                gobject.idle_add(self.pbar.set_fraction, self.pbar.get_fraction() + 0.0099)

        def data(self, msg):
                self.putcmd("data")
                (code, repl) = self.getreply()
                if self.debuglevel >0 : print>>stderr, "data:", (code, repl)
                if code != 354:
                    raise smtplib.SMTPDataError(code, repl)
                else:
                    q = smtplib.quotedata(msg)
                    if q[-2:] != smtplib.CRLF:
                        q = q + smtplib.CRLF
                    q = q + "." + smtplib.CRLF
                    self.split_msg(q)
                    (code, msg) = self.getreply()
                    if self.debuglevel > 0 :
                            print>>stderr, "data:", (code, msg)
                    return (code, msg)

        
 
class Smtp:

        def __init__(self, host, user=1, password=1, authenticate=False, message_dialog=None, port=25, pbar=None):
                self._host        = host
                self._port        = port                
                self._user        = user
                self._password    = password
                self._authenticate = authenticate
                self.message_dialog = message_dialog
                self._message     = None
                self._subject     = None
                self._from_addr   = None
                self._rcpt_to     = None               
                self._server      = None         
                self._attachments = []
                self.pbar = pbar
                self.connect()
		
        def connect(self):
                if all([self._host, self._port, self._user, self._password]):
                        try:    
                                self._server = MailServer(self._host, self._port, self.pbar)
                        except:
                                gobject.idle_add(self.message_dialog, _("Connection failed!"))
                        if self._authenticate:
                                try:
                                        self._server.login(self._user, self._password)        
                                except smtplib.SMTPException: 
                                        gobject.idle_add(self.message_dialog, _("Login failed!"))
 
        def close(self):                
                if self._server:
                        try:
                                self._server.quit()
                        except smtplib.SMTPException:
                                gobject.idle_add(self.message_dialog, _("Disconnection failed!"))
 
        def message(self, message):
                self._message = message
 
        def subject(self, subject):
                self._subject = subject
 
        def from_addr(self, email):
                self._from_addr = email
 
        def rcpt_to(self, email):
                self._rcpt_to = email
 
        def attach(self, file):
                if os.path.exists(file):
                        self._attachments.append(file)
 
        def load_attachments(self, m_message):
                for file in self._attachments:
                        part = MIMEBase('application', "octet-stream")
                        part.set_payload(open(file,"rb").read())
                        Encoders.encode_base64(part)
                        part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(file))
                        m_message.attach(part)
                return m_message
 
        def send(self, content_type='plain', charset='UTF-8'):
                if all([self._message, self._subject, self._from_addr, self._rcpt_to, self._server]):                                
                        m_message             = MIMEMultipart()
                        m_message['From']     = self._from_addr
                        m_message['To']       = self._rcpt_to
                        m_message['Date']     = formatdate(localtime=True)
                        m_message['Subject']  = self._subject
                        m_message['X-Mailer'] = "GTKRawGallery Mail Client"
                        m_message.attach(MIMEText(self._message, content_type, charset))
                        m_message = self.load_attachments(m_message)
                        try:
                                self._server.sendmail(self._from_addr, self._rcpt_to, m_message.as_string())
                        except smtplib.SMTPException:
                                gobject.idle_add(self.message_dialog, _("Email has not been sent"))
                                return False
                        return True
                return False
                                
